package multex; //MsgText.java

import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;



//Change history:
//2003-09-09  Knabe  MulTEx 5: Dynamic internationalization only by ResourceBundle
//                   parameter i_lineSeparator removed in all methods
//2003-05-22  Knabe  Enables dynamic internationalization. See appendMessageChain.
//2001-02-26  Knabe  Internationalization of message texts is now optional.
//2000-10-25  Knabe  appendMessageChain and appendMessageLine append to a
//                   StringBuffer rather than returning a String.
//2000-07-07  Knabe  extrahiert aus Klasse Msg


/** Services for getting the message text for exceptions.
  It handles exceptions <UL>
    <LI>with parameters (subclasses of {@link Exc})
    <LI>with cause chain and parameters (subclasses of {@link Failure})
    <LI>and standard Java exceptions (other subclasses of java.lang.Throwable)
  </UL>
  For normal message reporting you should use the services of one of the
  destination oriented reporting classes, e.g. {@link Msg} or {@link Awt}.

  <P>MulTEx 5: Parameter i_lineSeparator removed in all methods.
  Always uses the platform default.
  See the discussion in the description of class {@link Msg}.

  @author Christoph Knabe, Berlin, Copyright 1999-2003
*/
public class MsgText {


private static final String _className = MsgText.class.getName();

/**The name of the ResourceBundle used on static default internationalization.
    @see #setInternationalization(boolean)
*/
public  static final String resourceBundleName = "MsgText";

/**The String used by default to introduce the message of an exception,
 * which was causing the exception justly reported.
 * You can customize this by defining a cause marker in the localization
 * ResourceBundle under the key {@link #causeMarkerKey}.
 * */
public  static final String causeMarker = "CAUSE: ";

/**The key used to get the cause marker from a ResourceBundle.
 * It is the fully qualified name of the field {@link #causeMarker},
 * In the .properties file for a localization you should define it e.g.:
 * <PRE>multex.MsgText.causeMarker = Cause: </PRE>
 * */
public  static final String causeMarkerKey = _className + ".causeMarker";

/**If true, activates the static default internationalization of message texts.
    Further calls to
  {@link #appendMessageLine} or {@link #appendMessageChain} will deliver the
  localized message text for an exception instead of the default message
  text contained in the MultexException-object.
  The localized message text will be taken from the resource bundle
  with the name {@link #resourceBundleName}.

  If false, deactivates static internationalization. The Message text for a
  MultexException will be taken directly from the exception object itself.

  @since MulTEx 1c
*/
public static synchronized void setInternationalization(final boolean i_internationalization){
    _internationalization = i_internationalization;
    if(_internationalization){
    	_resourceBundle = _tryLoadMessages();
    }else{
    	_resourceBundle = null;
    }
}


/**Activates a specific static internationalization of message texts.
 * Further calls to
  {@link #appendMessageLine} or {@link #appendMessageChain} will deliver the
  localized message text for an exception instead of the default message
  text contained in the MultexException-object.

  @param i_resourceBundle How to localize exception message texts

  @throws IllegalArgumentException If null is provided as i_locale

  @since MulTEx 4

  @see #setInternationalization(boolean)
*/
public static synchronized void setInternationalization(final ResourceBundle i_resourceBundle)
throws IllegalArgumentException
{
	if(i_resourceBundle==null){throw new IllegalArgumentException("ResourceBundle must not be null");}
    _internationalization = true;
	_resourceBundle = i_resourceBundle;
}

/**Returns the actual ResourceBundle used for static internationalization
 * or null, if no one is provided.*/
public static ResourceBundle getResourceBundle(){return _resourceBundle;}

/**Appends the message for i_throwable with its causal chain.
  Appends an usually multi-line message, in which the first line is the
    message for i_throwable and the further lines are the messages in the
    chain of causing exceptions of i_throwable.
    See {@link #appendMessageLine} for a despription,
    how each exception is transformed into a message line.
  @param i_lineSeparator The String used to separate the lines of the message chain.
    If null is provided, defaults to {@link Failure#lineSeparator}.
  @deprecated since MulTEx 4 (2003-05-27). Use the more parameterized version instead.

public static void appendMessageChain(
  final StringBuffer io_destination,
  final Throwable i_throwable,
  final String i_lineSeparator
){
  appendMessageChain(
    io_destination, i_throwable, _resourceBundle, i_lineSeparator
  );
}
*/


/**Appends the message for i_throwable along with its causal chain.
    Preferably use {@link Msg#printMessages(StringBuffer,Throwable,ResourceBundle)} instead.
*/
protected static void appendMessageChain(
    final StringBuffer             io_destination,
    final Throwable                 i_throwable,
    final ResourceBundle            i_resourceBundle
){
	final ResourceBundle resourceBundle = i_resourceBundle==null ? getResourceBundle() : i_resourceBundle;
    final String causeSeparator
    = Failure.lineSeparator + _getCauseMarker(resourceBundle)
    ;
    if(i_throwable==null){
    	io_destination.append("null Throwable provided to ")
    	.append(_className).append(".appendMessageChain(...)");
    	return;
    }
    //Throwable-Meldungen eintragen:
    final String failureClassName = Failure.class.getName();
    boolean alreadyReported = false;
    for(Throwable t=i_throwable; t!=null; ){
        final Throwable cause = Failure.getCause(t);
    	
    	thisThrowable: {
	    	//Skip Failure without own info, as it is used as a tunneling 
	    	//unchecked exception:
		    if(t.getClass().getName().equals(failureClassName)){
		        final Failure failure = (Failure)t;
		        if( failure.getDefaultMessageTextPattern()==null
		            && failure.getParameters()==null 
		            && cause!=null
		        ){break thisThrowable;}
		    }
	
	        if(alreadyReported){
	            io_destination.append(causeSeparator);
	    	}
	    	
	    	_appendMessageLineDeletingPreviousOccurence(
				io_destination,	t,	resourceBundle
			);
	    	alreadyReported = true;
    	}//thisThrowable:
		t = cause;
    }
}

/**Appends the usually one-line message text for the exception
  i_throwable with inserted parameters to io_buffer.
  This variant deletes one previous occurence of the message line 
  in io_destination.
*/
private static void _appendMessageLineDeletingPreviousOccurence(
	final StringBuffer io_destination,
	final Throwable t,
	final ResourceBundle resourceBundle
){
	final int previousLength = io_destination.length();
	
	//appendMessageLine(io_destination, t, resourceBundle);
	//final String lastLine = io_destination.substring(previousLength); //substring since JDK 1.2
	final String lastLine = appendMessageLine(io_destination, t, resourceBundle);
	
	if(lastLine!=null){
		//platform exception without message text pattern, may be redundant in message chain
		final int firstOccurenceIndex = io_destination.indexOf(lastLine);
		if(firstOccurenceIndex<previousLength){ //There is a previous occurence of lastLine
			//replace it by "...":
			io_destination.replace(firstOccurenceIndex,firstOccurenceIndex+lastLine.length(),"..."); //replace since JDK 1.2
		}
	}
}


/**Appends the usually one-line message text for the exception
  i_throwable with inserted parameters to io_buffer.

  The message text pattern for the exception is taken from the mostly localized
  variant of the properties file "MsgText.properties" or helpwise from the
  default message text pattern in the exception object itself. A local message
  text pattern for the german language (de) in its swiss (CH) variant would be
  taken from file "MsgText_de_CH.properties", see the Java Tutorial
  <A HREF="http://java.sun.com/docs/books/tutorial/i18n/format/messageFormat.html">
  chapter on internationalization</A> at february 1999.
  <P>
  This file must contain definition lines of the form: <PRE>
     packageName.ExceptionName = message text pattern
  </PRE>or<PRE>
     packageName.ClassName$InnerExceptionName = message text pattern
  </PRE>with parameter substitutors in the message text pattern, e.g.:<PRE>
     CopyNP$CopyFailure = The file "{0}" could not be copied to "{1}"
  </PRE>
*/
public static void appendMessageLine(final StringBuffer io_destination, final Throwable i_throwable){
	appendMessageLine(io_destination, i_throwable, getResourceBundle());
}

/**Appends the usually one-line message text for the exception
 * i_throwable with inserted parameters to io_buffer, localizable.
 * <P>The message line format is produced according to the following case list:
 * <UL>
 *   <LI>MulTEx exceptions:
 *     <UL>
 *       <LI>textPattern & parameters: textPattern with parameters inserted
 *       <LI>textPattern, no parameters: textPattern unchanged
 *       <LI>no textPattern, with parameters: ClassName: getMessage()
 *       <LI>neither textPattern nor parameters: ClassName
 *     </UL>
 *   <LI>Standard Java exceptions:
  *     <UL>
 *       <LI>textPattern & detailMessage: textPattern: getMessage()
 *       <LI>textPattern, no detailMessage: textPattern unchanged
 *       <LI>no textPattern, with detailMessage: ClassName: getMessage()
 *       <LI>neither textPattern nor detailMessage: ClassName
 *     </UL>
 * </UL>
 *
 *
 * @param i_resourceBundle Source for the message text pattern for reporting i_throwable.
 * 			The text pattern is taken with the class name of i_throwable
 *          as key from i_resourceBundle.
 *          Controls the formatting of the message parameters, too, by containing a Locale.
 *          If null, falls back to no internationalization.
 * @return The appended line, if it was created by i_throwable.toString()
 * 			If not null, i_throwable had neither a text pattern, nor message parameters.
 *          Useful only for efficiently deleting redundant info in legacy string chain of platform exceptions.
 * @see #appendMessageLine(StringBuffer, Throwable) for details of parameter substitution
 * */
public static String appendMessageLine(
	final StringBuffer io_destination,
	final Throwable i_throwable,
	final ResourceBundle i_resourceBundle
) {
	//final ResourceBundle i_resourceBundle = i_resourceBundle==null ? getResourceBundle() : i_resourceBundle;
	  try {
//	    final String textPattern = getMessageTextPattern(i_throwable, resourceBundle);
//	    final Object[] parameters = _getParameters(i_throwable);
//	    if(parameters==null){ //no insertable exception parameters
//	      io_destination.append(textPattern);
//	    }else{ //with exception parameters:
//	      final Locale locale = resourceBundle==null ? null : resourceBundle.getLocale();
//		  format(io_destination, textPattern, parameters, locale);
//	    }
//	    if(!(i_throwable instanceof MultexException)){
//	      //other Java exceptions may have detailMessage:
//	      final String detailMessage = i_throwable.getMessage();
//	      if(detailMessage!=null && detailMessage.length()!=0){
//	        //Append standard Java detailMessage, separated by a colon
//	        io_destination.append(": ");
//	        io_destination.append(detailMessage);
//	      }
//	    }

	    final String textPattern = getMessageTextPattern(i_throwable, i_resourceBundle);
	    final Object[] parameters = _getParameters(i_throwable);
	    if(textPattern!=null && parameters!=null){ //full regular MulTEx case
	    	//Insert parameters into text pattern:
	        final Locale locale = i_resourceBundle==null ? null : i_resourceBundle.getLocale();
		    format(io_destination, textPattern, parameters, locale);
	    }else if(textPattern!=null && i_throwable instanceof MultexException){ 
	    	//MulTEx case without insertable exception parameters
	        io_destination.append(textPattern);
	    }else if(textPattern!=null){ //a standard exception with text from resource bundle
	        io_destination.append(textPattern);
	        final String detailMessage = i_throwable.getMessage();
	        if(detailMessage!=null){
		        io_destination.append(": ");
		        io_destination.append(detailMessage);
	        }
	    }else{ //no text pattern available
	    	//use Throwable.toString(), which uses reports in the format
	    	//   ClassName: Throwable.getMessage(),
	    	//which is suitably redefined for any MultexException, too:
	    	final String result = i_throwable.toString();
			io_destination.append(result);
			return result;
	    }
		return null;
	  } catch(final Exception ex){
	        System.err.println();
	        System.err.println(_className + ": WHEN PREPARING MESSAGE LINE OCCURRED EXCEPTION:");
	        ex.printStackTrace();
	        //io_destination.append(Failure.lineSeparator);
	    	final String result = i_throwable.toString();
			io_destination.append(result);
			return result;
	    }
	  //end try
}//appendMessageLine

/**Formats i_textPattern inserting the formatted elements of i_parameters
 * instead of the placeholders {0} to {9}.
 * This is a convenience method packing services of {@link java.text.MessageFormat}.
 * @param io_destination Where the result is appended to.
 * @param i_locale Used for localized formatting of i_parameters. If null, uses the default Locale.
 * */
/*package*/ static void format(
	final StringBuffer io_destination,
	final String i_textPattern,
	final Object[] i_parameters,
	final Locale i_locale
) {
	  final MessageFormat format = new MessageFormat(i_textPattern);
	  if(i_locale!=null){format.setLocale(i_locale);}
	  format.format(i_parameters, io_destination, null);
}

/**Gets a message text pattern for the exception i_throwable.
  The following sources are tried one after another with the class name
  of i_throwable as message text key: <OL>
    <LI>The language and country specific MsgText properties file,
        e.g. MsgText_en_UK.properties
    <LI>The language specific MsgText properties file,
        e.g. MsgText_en.properties
    <LI>The default MsgText properties file: MsgText.properties
    <LI>The default message text pattern from i_throwable, if it is a
        MultexException and not null or an empty String.
    <LI>null in all other cases.
  </OL>
*/
/*package*/ static String getMessageTextPattern(final Throwable i_throwable){
	return getMessageTextPattern(i_throwable, _resourceBundle);
}


/**Gets a message text pattern for the exception i_throwable.
  The following sources are tried one after another with the class name
  of i_throwable as message text key: <OL>
    <LI>The ResourceBundle i_resourceBundle,
    <LI>The default message text pattern from i_throwable, if it is a
        MultexException,
    <LI>null otherwise
  </OL>
*/
/*package*/ static String getMessageTextPattern(
	final Throwable i_throwable, final ResourceBundle i_resourceBundle
) {
    final String className = i_throwable.getClass().getName();
    final String defaultPattern = i_throwable instanceof MultexException ?
        ((MultexException)i_throwable).getDefaultMessageTextPattern() : null
    ;
	return _getResource(i_resourceBundle, className, defaultPattern);
}//_getMessageTextPattern

/**Returns the value for the resource with name i_key, helpwise i_defaultValue*/
private static String _getResource(
	final ResourceBundle i_resourceBundle,
	final String i_key,
	final String i_defaultValue)
{
	if(i_resourceBundle==null){return i_defaultValue;}
    try{
    	return i_resourceBundle.getString(i_key);
    }catch(java.util.MissingResourceException ex){
      return i_defaultValue;
    }
    //try
}//_getResource

/**Returns the parameters contained in i_throwable or null*/
private static Object[] _getParameters(final Throwable i_throwable){
  if(!(i_throwable instanceof MultexException)){return null;}
  return ((MultexException)i_throwable).getParameters();
}

/*debug
private static void _println(final String i_name, final Object[] i_array){
  System.err.print(i_name + ": ");
  for(int i=0; i<i_array.length; i++){
    System.err.print(i_array[i] + "  ");
  }
  System.err.println();
}
*/


/**The maximum number of index parameters, as allowed by
  java.text.MessageFormat
*/
private static final int _maxNumberOfIndexParams = 10;


//Message text access:

private static boolean        _internationalization = false;
private static ResourceBundle  _resourceBundle = null;
private static boolean        _loadMessagesTried = false;

/**If _internationalization is on, tries once to load the ResourceBundle
  resourceBundleName as message text file and returns it.
  If the loading fails, this method will return null,
  and will do nothing on further invocations.
*/
private static java.util.ResourceBundle _tryLoadMessages(){
  if(_loadMessagesTried){return null;}
  try{
    _loadMessagesTried = true;
    return java.util.ResourceBundle.getBundle(resourceBundleName);
  } catch(final java.util.MissingResourceException ex){ //resource not found
      System.err.println(ex);
      System.err.println(_className + ": Unable to load exception message texts"
        + " from the ResourceBundle\n" + resourceBundleName
      );
      return null;
    } catch(final Exception ex){ //not loadable
      ex.printStackTrace();
      return null;
    }
  //end try
}//_tryLoadMessages

private static String _getCauseMarker(final ResourceBundle i_resourceBundle){
	return _getResource(i_resourceBundle, causeMarkerKey, causeMarker);
}

/**Do not create objects of me!*/
private MsgText(){}

}//MsgText
